const util = require("../../utils/util");
let app = getApp();

Component({
  options: {
    addGlobalClass: true,
    multipleSlots: true
  },
  /**
   * 组件的属性列表
   */
  properties: {
    photoData: {
      type: Array,
      value: []
    },
    findPicShow: {
      type: Boolean,
      value: false
    }
  },

  /**
   * 组件的初始数据
   */
  data: {
    viewHeight: tt.getSystemInfoSync().windowHeight,
    photoData: [],
    activeDate: 0, //选中日期
    selectMore: false, //批量下载
    checkedGroup: [], //批量保存选择图片url
    imgArr: [],
    // 列表请求参数
    listData: [], // 列表数据
    column: 2, // 列数
    columnsHeights: [0, 0], // 每列高度
    // 页面展示数据参数
    pageIndex: 1,
    pageSize: 12,
  },

  /**
   * 组件的方法列表
   */
  methods: {
    // 吸顶
    scroll(e) {
      let query = tt.createSelectorQuery().in(this);
      let height = '';
      query.select('#topImg').boundingClientRect((rect) => {
        //获取到元素
        // console.log(6767, height,rect, e.detail.scrollTop)
        height = rect.height;
        if (e.detail.scrollTop >= height) {
          if (!this.data.ceilingShow) {
            this.setData({
              ceilingShow: true
            });
          }
        } else {
          if (this.data.ceilingShow) {
            this.setData({
              ceilingShow: false
            });
          }
        }
      }).exec();
    },
    // 当前页的数据处理
    setImgArr() {
      let {
        pageIndex,
        pageSize
      } = this.data;
      let start = (pageIndex - 1) * pageSize;
      let end = start + (pageSize - 1);
      const list = this.data.photoData[this.data.activeDate].photoList.slice(start, end);
      let newArr = [];
      let obj = {};
      // 屏宽
      const ww = tt.getSystemInfoSync().windowWidth;
      let queue = list.map((ele, i) => {
        // 获取图片高度
        let url = ele + '?x-oss-process=image/info';
        return new Promise((resolve) => {
          tt.request({
            url: url,
            success(res) {
              if (res.statusCode == 200) {
                let nowWidth = (ww - 20) * 0.98;
                // Orientation: 6 （顺时针翻转90度）
                // Orientation : 8 （逆时针翻转90度）
                let picW = '';
                let picH = '';
                if (res.data.Orientation && res.data.Orientation.value == 8 || res.data.Orientation && res.data.Orientation.value == 6) {
                  picW = res.data.ImageHeight.value;
                  picH = res.data.ImageWidth.value;
                } else {
                  picW = res.data.ImageWidth.value;
                  picH = res.data.ImageHeight.value;
                }
                obj.height = picH / (picW / nowWidth) / 2 + 5; //计算使用px 避免不同设备展示不同
                obj.height = obj.height.toFixed(2);
              } else {
                obj.height = '90';
              }
              obj.url = ele;
              obj.flag = true;
              // 用于预览图片
              obj.imgIndex = i + start;
              newArr.push(JSON.parse(JSON.stringify(obj)));
              resolve(newArr);
            }
          });
        });
      });
      Promise.all(queue).then((queueItem) => {
        console.log(queueItem[0], pageIndex,
        pageSize);
        this.setData({
          imgArr: queueItem[0]
        });
        this.getList();
      });
    },
    // 处理页面展示数据
    getList() {
      let {
        pageIndex,
        column,
        columnsHeights
      } = this.data;
      const columns = [];
      // 上一组的高度数据，用于计算偏移值
      const lastHeights = [...columnsHeights];
      // 获取数据
      const list = this.data.imgArr.sort((a, b) => {
        return a.imgIndex - b.imgIndex;
      });
      // 初始化当前屏数据
      for (let i = 0; i < column; i++) {
        columns.push([]);
      }
      // 遍历新数据，分配至各列
      for (let i = 0; i < list.length; i++) {
        const position = this.computePosition(columnsHeights);
        columns[position].push(list[i]);
        columnsHeights[position] += Number(list[i].height);
      }
      this.setData({
        [`listData[${pageIndex - 1}]`]: {
          columns,
          columnOffset: this.computeOffset(lastHeights)
        },
        pageIndex: pageIndex + 1,
        currentPage: pageIndex + 1,
        columnsHeights
      });
      console.log(this.data.listData, 7777);
    },
    /**
     * 获取最小高度列下标
     */
    computePosition(heights) {
      const min = Math.min(...heights);
      return heights.findIndex((item) => item === min);
    },
    /**
     * 计算偏移量
     */
    computeOffset(heights) {
      const max = Math.max(...heights);
      return heights.map((item) => max - item);
    },
    imgListBinderror(e) {
      let picInfo = e.currentTarget.dataset;
      this.data.listData[picInfo.pageindex].columns[picInfo.columnindex][picInfo.index].firstPic = '/img/noimg.png';
      this.setData({
        listData: this.data.listData
      });
    },
    onScrollLower() {
      // && this.data.listData.length > 0切换日期不调用
      if ((this.data.pageIndex - 1) * this.data.pageSize < this.data.photoData[this.data.activeDate].photoList.length && this.data.listData.length > 0) {
        this.setImgArr();
      }
    },
    // 切换图片日期
    dataChange(e) {
      // 不为当前触发
      if (this.data.activeDate != e.currentTarget.dataset.index) {
        let i = e.currentTarget.dataset.index;
        let arr = this.data.photoData[i].photoList;
        let copy = [];
        arr.map((ele, i) => {
          copy.push(ele);
        });
        this.setData({
          // 拍摄日期锚点
          topTimeId: 'tt' + i,
          timeId: 't' + i,
          bottomId: 'p1',
          picArr: copy,
          activeDate: i,
          // 清空页面
          listData: [],
          pageIndex: 1,
          pageSize: 12,
          currentPage: 0,
          // 列表请求参数
          columnsHeights: [0, 0] // 每列高度
        });
        this.closePro();
        this.setImgArr();
      }
    },
    // 预览图片
    openpreviewImg(e) {
      let that = this;
      let index = e.target.dataset.imgindex;
      let abbreviationPic = [],
        originalArr = [];
      that.data.photoData[that.data.activeDate].photoList.forEach((item) => {
        let item1 = {
          url: '',
          originalShow: false
        };
        item1.url = item + '?x-oss-process=image/quality,q_40';
        abbreviationPic.push(item1);
        originalArr.push(item);
      });
      that.setData({
        defImg: abbreviationPic[index].url, //当前图片
        preList: abbreviationPic, //缩略图列表
        originalArr: originalArr //原图列表
      });
      // 避免再次预览有个切换的过程
      setTimeout(function () {
        that.selectComponent("#previewComponent").showPreview();
      }, 400);

    },
    // 点击批量保存
    selectMore() {
      this.setData({
        selectMore: true,
        checkedGroup: []
      });
    },
    toFindPhoto() {this.triggerEvent('listener', 1);},
    // 批量保存选择图片
    checkRadio(e) {
      // console.log(e, 6789678)
      let picInfo = e.currentTarget.dataset;
      let currentItem = this.data.listData[picInfo.pageindex].columns[picInfo.columnindex][picInfo.index];
      if (picInfo.flag == true) {//未选中，修改选中项数组
        if (this.data.checkedGroup.length < 100) {
          this.data.listData[picInfo.pageindex].columns[picInfo.columnindex][picInfo.index].flag = false;
          this.data.checkedGroup.push(currentItem.url);
        } else {
          util.showTips('因照片较大，一次最多保存100张照片～');
        }
      } else {//未选中，修改原数组
        this.data.listData[picInfo.pageindex].columns[picInfo.columnindex][picInfo.index].flag = true;
        this.data.checkedGroup.forEach((el, i) => {
          if (currentItem.url == el) {
            this.data.checkedGroup.splice(i, 1);
          }
        });
      }
      this.setData({
        listData: this.data.listData,
        checkedGroup: this.data.checkedGroup
      });
    },
    // 保存图片权限判断
    photosAlbum() {
      var that = this;
      tt.getSetting({
        success(res) {
          if (res.authSetting['scope.writePhotosAlbum']) {
            // 已经获得授权，直接存储
            that.save();
          } else
          if (res.authSetting['scope.writePhotosAlbum'] === undefined) {
            // 第一次运行，授权未定义，可以直接保存，系统会一次性询问用户权限
            that.save();
          } else
          {
            // 用户拒绝授权后，打开设置页可以看到授权提示开关
            tt.openSetting({
              success(res) {// 用户授权
                if (res.authSetting['scope.writePhotosAlbum']) {
                  that.save();
                } else
                {// 用户拒绝授权
                  tt.showToast({
                    title: '权限不足',
                    duration: 3000,
                    icon: 'none'
                  });
                }
              },
              fail(res) {
                tt.showToast({
                  title: '设置失败',
                  duration: 3000,
                  icon: 'none'
                });
              }
            });
          }
        },
        fail(res) {
          console.log(res);
          tt.showToast({
            title: '设置失败',
            duration: 3000,
            icon: 'none'
          });
        }
      });
    },
    // 保存图片
    save(event) {
      if (this.data.checkedGroup && this.data.checkedGroup.length > 0) {
        const imageList = [];
        this.setData({
          progressShow: true,
          downloadNum: this.data.checkedGroup.length,
          completeNum: 0,
          downloadCompleted: false
        });
        // 循环数组
        for (let i = 0; i < this.data.checkedGroup.length; i++) {
          imageList.push(this.getTempPath(this.data.checkedGroup[i]));
        }
        console.log(imageList, 777);
        let index = 0;
        while (index < imageList.length) {
          // 将数据分割成多个promise数组
          Promise.all(imageList.slice(index, index += 8)).
          then((res) => {
            // 未取消保存
            if (this.data.progressShow) {
              res.forEach((item) => {
                tt.saveImageToPhotosAlbum({
                  filePath: item.tempFilePath,
                  success(res) {

                    // console.log(that.data.completeNum)
                  }, fail: function (err) {}
                });
              });
            }
          }).
          catch((err) => {
            reject(err);
          });
        }
      } else {
        tt.showToast({
          title: '请选择图片',
          duration: 3000,
          icon: 'none'
        });
      }
    },
    // 保存图片的延迟回调
    getTempPath(url) {
      let that = this;
      return new Promise((resolve, reject) => {
        tt.downloadFile({
          url: url,
          success: function (res) {
            that.setData({
              completeNum: that.data.completeNum + 1
            });
            console.log(that.data.completeNum);
            return resolve(res);
          },
          fail: function (err) {
            reject(url + JSON.stringify(err));
          }
        });
      });
    },
    // 关闭下载弹窗和多选，清空选择数据和状态
    closePro() {
      this.data.listData.forEach((item) => {
        item.columns.forEach((cItem) => {
          cItem.forEach((dItem) => {
            dItem.flag = true;
          });
        });
      });

      this.setData({
        progressShow: false,
        selectMore: false,
        listData: this.data.listData,
        checkedGroup: []
      });
    },
    // 下载完成
    downloadComplete() {
      // 最后一次下载
      if (this.data.completeNum == this.data.downloadNum) {
        this.setData({
          downloadCompleted: true
        });
      }
    },
    // 重置数据
    resetData() {
      this.setData({
        columnsHeights: [0, 0], // 每列高度
        activeDate: 0,
        listData: [], // 列表数据
        // 页面展示数据参数
        pageIndex: 1,
        pageSize: 12
      });
    }
  },
  observers: {
    'photoData': function (arr) {
      console.log(arr);
      if (this.data.photoData && this.data.photoData.length > 0) {
        // 重置
        this.resetData();
        this.setImgArr();
      } else {
        // 重置
        this.resetData();
      }
    }
  }
});